import os
import numpy as np
import imageio

import torch
import pytorch3d

from starter.utils import get_device, get_mesh_renderer, load_cow_mesh

from utils_yc import render_360

if __name__ == '__main__':

    out_d = 'output'
    if not os.path.exists(out_d): os.makedirs(out_d)

    device = get_device()
    cow_path = 'data/cow.obj'

    # print(device)

    # renderer
    image_size = 256
    renderer = get_mesh_renderer(image_size=image_size)

    # Get the vertices, faces, and textures.
    vertices, faces = load_cow_mesh(cow_path)
    vertices = vertices.unsqueeze(0)  # (N_v, 3) -> (1, N_v, 3)
    faces = faces.unsqueeze(0)  # (N_f, 3) -> (1, N_f, 3)
    textures = torch.ones_like(vertices)  # (1, N_v, 3)
    color=[0.7, 0.7, 1]
    textures = textures * torch.tensor(color)  # (1, N_v, 3)
    mesh = pytorch3d.structures.Meshes(
        verts=vertices,
        faces=faces,
        textures=pytorch3d.renderer.TexturesVertex(textures),
    )
    mesh = mesh.to(device)

    rend_imgs = render_360(mesh, renderer)

    imageio.mimsave(f'{out_d}/q1.1_cow_gif.gif', rend_imgs, fps=15)