import os
import numpy as np
import imageio

import torch
import pytorch3d

from starter.utils import get_device, get_mesh_renderer, load_cow_mesh

from utils_yc import render_360

if __name__ == '__main__':

    out_d = 'output'
    if not os.path.exists(out_d): os.makedirs(out_d)

    device = get_device()
    cow_path = 'data/cow.obj'

    # print(device)

    # renderer
    image_size = 256
    renderer = get_mesh_renderer(image_size=image_size)

    # Get the vertices, faces, and textures.
    vertices, faces = load_cow_mesh(cow_path)
    vertices = vertices.unsqueeze(0)  # (N_v, 3) -> (1, N_v, 3)
    faces = faces.unsqueeze(0)  # (N_f, 3) -> (1, N_f, 3)
    # textures = torch.ones_like(vertices)  # (1, N_v, 3)

    N = vertices.shape[1]
    color1 = torch.tensor([0.3, 0.5, 0.7]).repeat(N, 1)
    color2 = torch.tensor([0.7, 0.5, 0.3]).repeat(N, 1)

    z_min = vertices[:, :, 2].min()
    z_max = vertices[:, :, 2].max()
    alpha = (vertices[0, :, 2] - z_min) / (z_max - z_min)
    alpha = alpha.unsqueeze(1)
    color = alpha * color2 + (1-alpha) * color1

    # textures = textures * torch.tensor(color)  # (1, N_v, 3)
    textures = color.unsqueeze(0)
    mesh = pytorch3d.structures.Meshes(
        verts=vertices,
        faces=faces,
        textures=pytorch3d.renderer.TexturesVertex(textures),
    )
    mesh = mesh.to(device)

    rend_imgs = render_360(mesh, renderer)

    imageio.mimsave(f'{out_d}/q3_cow.gif', rend_imgs, fps=15)