"""
Usage:
    python -m starter.camera_transforms --image_size 512
"""
import argparse
import math

import matplotlib.pyplot as plt
import pytorch3d
import torch

from starter.utils import get_device, get_mesh_renderer

def render_textured_cow(
    cow_path="data/cow.obj",
    image_size=256,
    R_relative=[[1, 0, 0], [0, 1, 0], [0, 0, 1]],
    T_relative=[0, 0, 0],
    device=None,
):
    if device is None:
        device = get_device()
    meshes = pytorch3d.io.load_objs_as_meshes([cow_path]).to(device)
    R_relative = torch.tensor(R_relative).float()
    T_relative = torch.tensor(T_relative).float()
    R = R_relative @ torch.tensor([[1.0, 0, 0], [0, 1, 0], [0, 0, 1]])
    T = R_relative @ torch.tensor([0.0, 0, 3]) + T_relative
    renderer = get_mesh_renderer(image_size=image_size)
    cameras = pytorch3d.renderer.FoVPerspectiveCameras(
        R=R.unsqueeze(0), T=T.unsqueeze(0), device=device,
    )
    lights = pytorch3d.renderer.PointLights(location=[[0, 0.0, -3.0]], device=device,)
    rend = renderer(meshes, cameras=cameras, lights=lights)
    return rend[0, ..., :3].cpu().numpy()


if __name__ == "__main__":
    cow_path = 'data/cow.obj'
    img_size = 256

    device = get_device()
    renderer = get_mesh_renderer(image_size=img_size)
    lights = pytorch3d.renderer.PointLights(location=[[0, 0.0, -3.0]], device=device,)

    # read mesh
    meshes = pytorch3d.io.load_objs_as_meshes([cow_path]).to(device)

    # img1
    R_rel = pytorch3d.transforms.axis_angle_to_matrix(torch.tensor([[0, 0, 270 * math.pi/180]])).squeeze(0)
    T_rel = torch.tensor([0, 0, 0])

    print('q4-1: ')
    print(R_rel)
    print(T_rel)
    R = R_rel @ torch.tensor([[1.0, 0, 0], [0, 1, 0], [0, 0, 1]])
    T = R_rel @ torch.tensor([0.0, 0, 3]) + T_rel

    cameras = pytorch3d.renderer.FoVPerspectiveCameras(
        R=R.unsqueeze(0), T=T.unsqueeze(0), device=device,
    )

    rend = renderer(meshes, cameras=cameras, lights=lights)[0, ..., :3].cpu().numpy()
    plt.imsave('output/q4-1.png', rend)

    # img2
    R_rel = pytorch3d.transforms.axis_angle_to_matrix(torch.tensor([[0., 0, 0]])).squeeze(0)
    T_rel = torch.tensor([0., 0., 2.])

    print('q4-2: ')
    print(R_rel)
    print(T_rel)
    R = R_rel @ torch.tensor([[1.0, 0, 0], [0, 1, 0], [0, 0, 1]])
    T = R_rel @ torch.tensor([0.0, 0, 3]) + T_rel

    cameras = pytorch3d.renderer.FoVPerspectiveCameras(
        R=R.unsqueeze(0), T=T.unsqueeze(0), device=device,
    )

    rend = renderer(meshes, cameras=cameras, lights=lights)[0, ..., :3].cpu().numpy()
    plt.imsave('output/q4-2.png', rend)

    # img3 ??
    R_rel = pytorch3d.transforms.axis_angle_to_matrix(torch.tensor([[0., 0, 0]])).squeeze(0)
    T_rel = torch.tensor([2/4, -2/4, 0]) # move cow: left (+x), down (-y), closer (-z)

    print('q4-3: ')
    print(R_rel)
    print(T_rel)
    R = R_rel @ torch.tensor([[1.0, 0, 0], [0, 1, 0], [0, 0, 1]])
    T = R_rel @ torch.tensor([0.0, 0, 3]) + T_rel

    cameras = pytorch3d.renderer.FoVPerspectiveCameras(
        R=R.unsqueeze(0), T=T.unsqueeze(0), device=device,
    )

    rend = renderer(meshes, cameras=cameras, lights=lights)[0, ..., :3].cpu().numpy()
    plt.imsave('output/q4-3.png', rend)

    R_rel = pytorch3d.transforms.axis_angle_to_matrix(torch.tensor([[0., 90 * math.pi / 180, 0]])).squeeze(0)
    T_rel = torch.inverse(R_rel) @ torch.tensor([0.0, 0, 3]) + torch.tensor([0.0, 0, 3])
    print('q4-4: ')
    print(R_rel)
    print(T_rel)
    
    R = R_rel @ torch.tensor([[1.0, 0, 0], [0, 1, 0], [0, 0, 1]])
    T = R_rel @ torch.tensor([0.0, 0, 3]) + T_rel

    cameras = pytorch3d.renderer.FoVPerspectiveCameras(
        R=R.unsqueeze(0), T=T.unsqueeze(0), device=device,
    )

    rend = renderer(meshes, cameras=cameras, lights=lights)[0, ..., :3].cpu().numpy()
    plt.imsave('output/q4-4.png', rend)
