"""
Sample code to render various representations.

Usage:
    python -m starter.render_generic --render point_cloud  # 5.1
    python -m starter.render_generic --render parametric  --num_samples 100  # 5.2
    python -m starter.render_generic --render implicit  # 5.3
"""
import argparse
import pickle

import matplotlib.pyplot as plt
import mcubes
import numpy as np
import pytorch3d
import torch

from starter.utils import get_device, get_mesh_renderer, get_points_renderer
from starter.utils import unproject_depth_image

from utils_yc import get_cam

import imageio
import math

if __name__ == "__main__":
    device = get_device()
    renderer = get_points_renderer(
        image_size=256, background_color=(1, 1, 1)
    )

    dist = 8
    elev = 10
    R, T = pytorch3d.renderer.look_at_view_transform(dist, elev, 0)
    # R_z = pytorch3d.transforms.axis_angle_to_matrix(torch.tensor([[45. * math.pi / 180, 0, 0.]])).squeeze(0)
    R_z = pytorch3d.transforms.axis_angle_to_matrix(torch.tensor([[0. * math.pi / 180, 0, 0.]])).squeeze(0)
    # R, T = R_z @ R[0], R_z @ T[0]
    R = R_z @ R[0]
    T = T[0]

    R_torus = 3
    r_torus = 1
    num_samples = 1000
    theta = torch.linspace(0, 2 * np.pi, num_samples)
    phi = torch.linspace(0, 2 * np.pi, num_samples)
    # Densely sample phi and theta on a grid
    Theta, Phi = torch.meshgrid(phi, theta)

    x = (R_torus + r_torus * torch.cos(Theta)) * torch.cos(Phi)
    y = (R_torus + r_torus * torch.cos(Theta)) * torch.sin(Phi)
    z = r_torus * torch.sin(Theta)

    points = torch.stack((x.flatten(), y.flatten(), z.flatten()), dim=1)
    points -= points.mean(0)
    color = (points - points.min()) / (points.max() - points.min())

    pcd = pytorch3d.structures.Pointclouds(points=points.unsqueeze(0), features=color.unsqueeze(0)).to(device)

    azims = torch.linspace(0, 360, 36)
    # azims = [60]

    rend_list = []
    R_list = []
    T_list = []
    for azim in azims:

        # look_at_view
        R, T = pytorch3d.renderer.cameras.look_at_view_transform(dist, elev, azim)
        cam = cameras = pytorch3d.renderer.FoVPerspectiveCameras(R=R, T=T, fov=60, device=device)
        # cam = get_cam(R, T, azim, device)
        R_list.append(R.squeeze(0))
        T_list.append(T.squeeze(0))

        rend = renderer(pcd, cameras=cam).cpu().numpy()[0, ..., :3]

        rend_all = (rend * 255).astype(np.uint8)
        rend_list.append(rend_all)

    # R1, R2 = R_list[:2]
    # Ry = pytorch3d.transforms.axis_angle_to_matrix(torch.tensor([[0. , 10.2857 * math.pi / 180, 0.]])).squeeze(0)
    # Ry @ R1 == R2
    # T1, T2 = T_list[:2]
    # T1 == T2

    imageio.mimsave(f'output/q5.2-torus.gif', rend_list, fps=15)