import os
import numpy as np
import imageio

import torch
import pytorch3d


from pytorch3d.vis.plotly_vis import plot_scene, AxisArgs

from starter.utils import get_device, get_mesh_renderer, load_cow_mesh

from utils_yc import render_360
import math


deg2rad =  math.pi / 180

def process_dolphin(dolphin_mesh):
    device = dolphin_mesh.device
    verts = dolphin_mesh.verts_list()[0]
    verts = (verts - verts.min()) / (verts.max() - verts.min())
    verts = (verts - verts.mean(0))
    dolphin_mesh._verts_list = [verts]

    scale = 1.0
    verts *= scale

    R = pytorch3d.transforms.axis_angle_to_matrix(torch.tensor([[0, 0, 180 * deg2rad]])).squeeze(0).to(device)
    # R = pytorch3d.transforms.axis_angle_to_matrix(torch.tensor([[0 * deg2rad, 90 * deg2rad, 0]])).squeeze(0).to(device)
    T = torch.tensor([0, -0.7, -0.2], device=device).unsqueeze(0).to(device)

    verts = verts @ R
    # verts = rot_x_90.transform_points(verts.unsqueeze(0)).squeeze(0)
    verts += T

    dolphin_mesh._verts_list = [verts]

    return dolphin_mesh

def process_cat(cat_mesh):
    device = cat_mesh.device
    verts = cat_mesh.verts_list()[0]
    verts = (verts - verts.min()) / (verts.max() - verts.min())
    verts = (verts - verts.mean(0))

    scale = 0.8
    verts *= scale

    R = pytorch3d.transforms.axis_angle_to_matrix(torch.tensor([[0 * deg2rad, 0, 0]])).squeeze(0).to(device)
    # R = pytorch3d.transforms.axis_angle_to_matrix(torch.tensor([[0 * deg2rad, 90 * deg2rad, 0]])).squeeze(0).to(device)
    T = torch.tensor([0, -0.1, -0.2], device=device).unsqueeze(0).to(device)

    verts = verts @ R
    # verts = rot_x_90.transform_points(verts.unsqueeze(0)).squeeze(0)
    verts += T


    cat_mesh._verts_list = [verts]
    return cat_mesh

def process_table(table_mesh):

    from pytorch3d.transforms import RotateAxisAngle
    

    device = table_mesh.device

    verts = table_mesh.verts_list()[0]
    verts = (verts - verts.min()) / (verts.max() - verts.min())
    verts = (verts - verts.mean(0, keepdim=True))

    scale = 3
    verts *= scale

    R = pytorch3d.transforms.axis_angle_to_matrix(torch.tensor([[0 * deg2rad, 0, 0]])).squeeze(0).to(device)
    # R = pytorch3d.transforms.axis_angle_to_matrix(torch.tensor([[0 * deg2rad, 90 * deg2rad, 0]])).squeeze(0).to(device)
    T = torch.tensor([0, 0.5, -0.4], device=device).unsqueeze(0).to(device)

    verts = verts @ R
    # verts = rot_x_90.transform_points(verts.unsqueeze(0)).squeeze(0)
    verts += T

    table_mesh._verts_list = [verts]
    return table_mesh

if __name__ == '__main__':

    out_d = 'output'
    if not os.path.exists(out_d): os.makedirs(out_d)

    device = get_device()
    cat_path = 'data/cat/cat.obj'
    dolphin_path = 'data/dolphin/dolphin.obj'
    # table_path = 'data/table/table.obj'
    table_path = 'data/bed/bed.obj'
    img_size = 256

    device = get_device()
    renderer = get_mesh_renderer(image_size=img_size)
    lights = pytorch3d.renderer.PointLights(location=[[0, 0.0, -3.0]], device=device,)

    # read mesh
    cat_mesh = pytorch3d.io.load_objs_as_meshes([cat_path]).to(device)
    dolphin_mesh = pytorch3d.io.load_objs_as_meshes([dolphin_path]).to(device)
    table_mesh = pytorch3d.io.load_objs_as_meshes([table_path]).to(device)

    cat_mesh = process_cat(cat_mesh)
    dolphin_mesh = process_dolphin(dolphin_mesh)
    table_mesh = process_table(table_mesh)


    # mesh_all = pytorch3d.structures.join_meshes_as_scene([cat_mesh, dolphin_mesh])

    # cam
    dist = 3
    elev = 10

    # rend = renderer(cat_mesh, cameras=cams).cpu().numpy()[0, ..., :3]
    mesh_all = pytorch3d.structures.join_meshes_as_scene([cat_mesh, dolphin_mesh, table_mesh])

    azims = torch.linspace(0, 360, 36)

    rend_all = []
    for azim in azims:
        R, T = pytorch3d.renderer.look_at_view_transform(dist, elev, azim)
        R = R.to(device)
        T = T.to(device)
        R_rel = pytorch3d.transforms.axis_angle_to_matrix(torch.tensor([[90 * deg2rad, 0, 0]])).squeeze(0).to(device)
        R = R_rel @ R

        cams = pytorch3d.renderer.FoVPerspectiveCameras(R=R, T=T, device=device)

        rend_scene = renderer(mesh_all, cameras=cams).cpu().numpy()[0, ..., :3]
        rend_scene = (rend_scene * 255).astype(np.uint8)
        rend_all.append(rend_scene)

    # fig = plot_scene({
    #     "fig": {
    #     "cat_mesh": cat_mesh,
    #     "dolphin_mesh": dolphin_mesh,
    #     "table_mesh": table_mesh,
    #     # "mesh_all": mesh_all,
    #     "camera": cams,
    #     }
    #     }, axis_args=AxisArgs(showgrid=True, backgroundcolor="#ccc"))
    # fig.show()

    # imageio.imwrite(f'{out_d}/q6.png', rend_scene)
    imageio.mimsave(f'{out_d}/q6.gif', rend_all, fps=15)