import numpy as np
import torch
import pytorch3d

from starter.utils import get_device

import math

def init_mesh(verts, faces, color=[0.7, 0.7, 1], device=None):
    if device == None:
        device = get_device()

    if verts.dim() == 2:
        verts = verts.unsqueeze(0)  # (N_v, 3) -> (1, N_v, 3)
    if faces.dim() == 2:
        faces = faces.unsqueeze(0)  # (N_f, 3) -> (1, N_f, 3)
    textures = torch.ones_like(verts)  # (1, N_v, 3)

    # color=[0.7, 0.7, 1]
    textures = textures * torch.tensor(color)  # (1, N_v, 3)

    mesh = pytorch3d.structures.Meshes(
        verts=verts,
        faces=faces,
        textures=pytorch3d.renderer.TexturesVertex(textures),
    )
    mesh = mesh.to(device)
    return mesh


def render_360(mesh, renderer, elev=0):
    device = mesh.device

    # get cameras
    dist = 3
    # elev = 0
    azim_list = torch.linspace(-180, 180, 36)

    # Place a point light in front of the cow.
    lights = pytorch3d.renderer.PointLights(location=torch.tensor([[0, 0, -3]]), device=device)

    imgs = []
    for azim in azim_list:
        R, T = pytorch3d.renderer.cameras.look_at_view_transform(dist, elev, azim)
        
        cameras = pytorch3d.renderer.FoVPerspectiveCameras(
        R=R, T=T, fov=60, device=device
        )

        # Place a point light at camera's position
        loc_light = torch.inverse(R)[0] @ -T[0]
        lights = pytorch3d.renderer.PointLights(location=loc_light.unsqueeze(0), device=device)

        rend = renderer(mesh, cameras=cameras, lights=lights)
        rend = (rend.cpu().numpy()[0, ..., :3] * 255).astype(np.uint8)

        imgs.append(rend)

        # imageio.imsave('img.png', imgs[0][0, ..., :-3].cpu().numpy())
        # import pdb; pdb.set_trace()
    return imgs


def get_cam(R0, T0, azim, device):
    R_rel = pytorch3d.transforms.axis_angle_to_matrix(torch.tensor([[0., azim * math.pi / 180., 0]])).squeeze(0)
    R = (R_rel @ R0).unsqueeze(0)
    # T = (R_rel @ T0).unsqueeze(0)
    # T = (torch.inverse(R_rel) @ T0) + T0 #
    T = T0.unsqueeze(0)
    cam = pytorch3d.renderer.FoVPerspectiveCameras(R=R, T=T, device=device)
    return cam